@@ -6,12 +6,11 @@
 
 #include "mozilla/Logging.h"
 
-#include <algorithm>
-
 #include "base/process_util.h"
 #include "GeckoProfiler.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "mozilla/FileUtils.h"
+#include "mozilla/GeckoTrace.h"
 #include "mozilla/LateWriteChecks.h"
 #include "mozilla/Mutex.h"
 #include "mozilla/StaticPtr.h"
@@ -27,6 +26,7 @@
 #include "NSPRLogModulesParser.h"
 #include "nsXULAppAPI.h"
 #include "LogCommandLineHandler.h"
+#include "fmt/format.h"
 
 #include "prenv.h"
 #ifdef XP_WIN
@@ -41,7 +41,7 @@
 // NB: Amount determined by performing a typical browsing session and finding
 //     the maximum number of modules instantiated, and padding up to the next
 //     power of 2.
-const uint32_t kInitialModuleCount = 256;
+const uint32_t kInitialModuleCount = 1024;
 // When rotate option is added to the modules list, this is the hardcoded
 // number of files we create and rotate.  When there is rotate:40,
 // we will keep four files per process, each limited to 10MB.  Sum is 40MB,
@@ -73,12 +73,6 @@ void log_print(const LogModule* aModule, LogLevel aLevel, TimeStamp* aStart,
 
 }  // namespace detail
 
-LogLevel ToLogLevel(int32_t aLevel) {
-  aLevel = std::min(aLevel, static_cast<int32_t>(LogLevel::Verbose));
-  aLevel = std::max(aLevel, static_cast<int32_t>(LogLevel::Disabled));
-  return static_cast<LogLevel>(aLevel);
-}
-
 static const char* ToLogStr(LogLevel aLevel) {
   switch (aLevel) {
     case LogLevel::Error:
@@ -585,12 +579,43 @@ class LogModuleManager {
              va_list aArgs) MOZ_FORMAT_PRINTF(4, 0) {
     Print(aName, aLevel, nullptr, "", aFmt, aArgs);
   }
+  void PrintFmt(const char* aName, LogLevel aLevel, fmt::string_view aFmt,
+                fmt::format_args aArgs) {
+    PrintFmt(aName, aLevel, nullptr, "", aFmt, aArgs);
+  }
+
+  void PrintFmt(const char* aName, LogLevel aLevel, const TimeStamp* aStart,
+                const char* aPrepend, fmt::string_view aFmt,
+                fmt::format_args aArgs) {
+    AutoSuspendLateWriteChecks suspendLateWriteChecks;
+    const size_t kBuffSize = 1024;
+    char buff[kBuffSize];
+
+    char* buffToWrite = buff;
+    UniquePtr<char[]> allocatedBuff;
+    size_t charsWritten;
+
+    auto [out, size] = fmt::vformat_to_n(buff, kBuffSize - 1, aFmt, aArgs);
+    *out = '\0';
+    charsWritten = size;
+
+    // We may have maxed out, allocate a buffer and re-format
+    if (charsWritten > kBuffSize) {
+      allocatedBuff = MakeUnique<char[]>(charsWritten + 1);  // + final \0
+      auto [out, size] =
+          fmt::vformat_to_n(allocatedBuff.get(), charsWritten, aFmt, aArgs);
+      MOZ_ASSERT(size == charsWritten);
+      *out = '\0';
+      buffToWrite = allocatedBuff.get();
+    }
+    ++charsWritten;  // + final \0
+    ActuallyLog(aName, aLevel, aStart, aPrepend, buffToWrite, charsWritten);
+  }
 
   void Print(const char* aName, LogLevel aLevel, const TimeStamp* aStart,
              const char* aPrepend, const char* aFmt, va_list aArgs)
       MOZ_FORMAT_PRINTF(6, 0) {
     AutoSuspendLateWriteChecks suspendLateWriteChecks;
-    long pid = static_cast<long>(base::GetCurrentProcId());
     const size_t kBuffSize = 1024;
     char buff[kBuffSize];
 
@@ -616,7 +641,13 @@ class LogModuleManager {
       buffToWrite = allocatedBuff.get();
       charsWritten = strlen(buffToWrite);
     }
+    ActuallyLog(aName, aLevel, aStart, aPrepend, buffToWrite, charsWritten);
+  }
 
+  void ActuallyLog(const char* aName, LogLevel aLevel, const TimeStamp* aStart,
+                   const char* aPrepend, const char* aLogMessage,
+                   size_t aLogMessageSize) {
+    long pid = static_cast<long>(base::GetCurrentProcId());
     if (profiler_thread_is_being_profiled_for_markers()) {
       struct LogMarker {
         static constexpr Span<const char> MarkerTypeName() {
@@ -648,12 +679,12 @@ class LogModuleManager {
                   : MarkerTiming::InstantNow(),
            MarkerStack::MaybeCapture(mCaptureProfilerStack)},
           LogMarker{}, ProfilerString8View::WrapNullTerminatedString(aName),
-          ProfilerString8View::WrapNullTerminatedString(buffToWrite));
+          ProfilerString8View::WrapNullTerminatedString(aLogMessage));
     }
 
     // Determine if a newline needs to be appended to the message.
     const char* newline = "";
-    if (charsWritten == 0 || buffToWrite[charsWritten - 1] != '\n') {
+    if (aLogMessageSize == 0 || aLogMessage[aLogMessageSize - 1] != '\n') {
       newline = "\n";
     }
 
@@ -690,10 +721,10 @@ class LogModuleManager {
       if (!mIsRaw) {
         fprintf_stderr(out, "%s[%s %ld: %s]: %s/%s %s%s", aPrepend,
                        nsDebugImpl::GetMultiprocessMode(), pid,
-                       currentThreadName, ToLogStr(aLevel), aName, buffToWrite,
+                       currentThreadName, ToLogStr(aLevel), aName, aLogMessage,
                        newline);
       } else {
-        fprintf_stderr(out, "%s%s%s", aPrepend, buffToWrite, newline);
+        fprintf_stderr(out, "%s%s%s", aPrepend, aLogMessage, newline);
       }
     } else {
       if (aStart) {
@@ -717,7 +748,7 @@ class LogModuleManager {
             start.tm_hour, start.tm_min, start.tm_sec, start.tm_usec,
             now.tm_hour, now.tm_min, now.tm_sec, now.tm_usec,
             duration.ToMilliseconds(), nsDebugImpl::GetMultiprocessMode(), pid,
-            currentThreadName, ToLogStr(aLevel), aName, buffToWrite, newline);
+            currentThreadName, ToLogStr(aLevel), aName, aLogMessage, newline);
       } else {
         PRExplodedTime now;
         PR_ExplodeTime(PR_Now(), PR_GMTParameters, &now);
@@ -727,7 +758,7 @@ class LogModuleManager {
                        aPrepend, now.tm_year, now.tm_month + 1, now.tm_mday,
                        now.tm_hour, now.tm_min, now.tm_sec, now.tm_usec,
                        nsDebugImpl::GetMultiprocessMode(), pid,
-                       currentThreadName, ToLogStr(aLevel), aName, buffToWrite,
+                       currentThreadName, ToLogStr(aLevel), aName, aLogMessage,
                        newline);
       }
     }
@@ -864,6 +895,13 @@ void LogModule::SetLevel(LogLevel level) {
   if (strstr(mName, "::")) {
     set_rust_log_level(mName, static_cast<uint8_t>(level));
   }
+
+  // The following enables the propagation of runtime-set log levels (for
+  // example configured via `about:logging`) from the Mozilla logging system to
+  // OpenTelemetry's internal logging mechanism.
+  if (strcmp(mName, "opentelemetry") == 0) {
+    gecko_trace::SetOpenTelemetryInternalLogLevel(level);
+  }
 }
 
 void LogModule::Init(int argc, char* argv[]) {
@@ -902,6 +940,14 @@ void LogModule::Printv(LogLevel aLevel, const TimeStamp* aStart,
   sLogModuleManager->Print(Name(), aLevel, aStart, "", aFmt, aArgs);
 }
 
+void LogModule::PrintvFmt(LogLevel aLevel, fmt::string_view aFmt,
+                          fmt::format_args aArgs) const {
+  MOZ_ASSERT(sLogModuleManager != nullptr);
+
+  // Forward to LogModule manager w/ level and name
+  sLogModuleManager->PrintFmt(Name(), aLevel, aFmt, aArgs);
+}
+
 }  // namespace mozilla
 
 extern "C" {
